/*****************************************************************************/
/*
  animate.c: animate a data file.
*/
/*****************************************************************************/
/* This version flips X and angles to get lighting right */
#define XFLIP -
#define XXFLIP 

/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "drawstuff.h" /* Graphics stuff */
#include "drawstuff-cga.h" /* CGA stuff to make things clearer */

#include "../useful/mrdplot/mrdplot.h"
#include "main.h"
#include "main2.h"

/*****************************************************************************/

enum channels {
  TIME=0,
  SS_X,
  SS_ANGLE,
  POLE_TOP_X,
  POLE_TOP_Y,
  POLE_TOP_Z,
  N_CHANNELS,
};

#define CART_SIZE (0.1)    // width of cart

#define POLE_LENGTH (1.0)     // length of link
#define POLE_WIDTH (0.05)    // width of link

/*****************************************************************************/

SIM sim;

MRDPLOT_DATA *d;

int channels[N_CHANNELS];

/*****************************************************************************/
/*****************************************************************************/

double lookup_value( MRDPLOT_DATA *d, int point, int channel )
{
  if ( channels[channel] < 0 || channels[channel] >= d->n_channels )
    {
      fprintf( stderr, "Bad channel: %d %d\n", channel, channels[channel] );
      exit( -1 );
    }
  return d->data[point*d->n_channels + channels[channel]];
}

/********************************************************************/
// start(): set things up

static void start()
{
  // set up view point
  /*
  static float xyz[3] = {0.0f,1.4f,0.8f};
  static float hpr[3] = {-90.0000f,0.0000f,0.0000f};
  */
  static float xyz[3] = {1.4f,0.0f,0.8f};
  static float hpr[3] = {-180.0000f,0.0000f,0.0000f};
  dsSetViewpoint (xyz,hpr);
}

/********************************************************************/
// called when a key pressed

static void command (int cmd)
{
  // don't handle user input yet.
  dsPrint ("received command %d (`%c')\n",cmd,cmd);
}

/*****************************************************************************/

static void display (int pause)
{
  static int index = 0;
  float f;
  float center[3];
  float R[12];
  int i;
  float cart_sides[3] = { CART_SIZE, CART_SIZE, CART_SIZE };
  float pole_sides[3] = { POLE_WIDTH, POLE_WIDTH, POLE_LENGTH };

  /* Markers */
  // set up identity matrix
  for ( i = 0; i < 12; i++ )
    R[i] = 0;
  R[0] = R[5] = R[10] = 1.0;

  /* Cart */
  center[XX] = 0;
  center[YY] = lookup_value( d, index, SS_X );
  center[ZZ] = 0;
  dsSetTexture (DS_WOOD);
  dsSetColor (0,0,1);
  dsDrawBox( center, R, cart_sides );

  /* Pole */
  center[XX] = 0;
  center[YY] = 0.5*(lookup_value( d, index, SS_X )
		    + lookup_value( d, index, POLE_TOP_Y ) );
  center[ZZ] = 0.5*lookup_value( d, index, POLE_TOP_Z );
  for ( i = 0; i < 12; i++ )
    R[i] = 0;
  R[0] = 1;
  R[5] = cos( lookup_value( d, index, SS_ANGLE ) );
  R[10] = R[5];
  R[6] = -sin( lookup_value( d, index, SS_ANGLE ) );
  R[9] = - R[6];
  dsSetTexture (DS_NONE);
  dsSetColor (1,0,0);
  dsDrawBox( center, R, pole_sides );

  /* Delay */
#ifdef WIN32
  for( f = 0.0; f < 7000000.0; f += 1.0 )
    ;
#else
  // for( f = 0.0; f < 12000000.0; f += 1.0 )
     for( f = 0.0; f < 2000000.0; f += 1.0 )
    ;
#endif

  index += 1;
  if ( index > d->n_points )
    index = 0;
}

/*****************************************************************************/
/********************************************************************/

int main (int argc, char **argv)
{
  char *filename;
  int i;
  dsFunctions fn; 

  if ( argc > 1 )
    filename = strdup( argv[1] );
  else
    filename = last_data();
  
  printf( "Reading: %s\n", filename );

  d = read_mrdplot( filename );

  for( i = 0; i < N_CHANNELS; i++ )
    channels[i] = -1;

  channels[TIME] = find_channel( "time", d );
  channels[SS_X] = find_channel( "x", d );
  channels[SS_ANGLE] = find_channel( "angle", d );
  channels[POLE_TOP_X] = find_channel( "pole_x", d );
  channels[POLE_TOP_Y] = find_channel( "pole_y", d );
  channels[POLE_TOP_Z] = find_channel( "pole_z", d );

  for( i = 0; i < N_CHANNELS; i++ )
    printf( "%d %d\n", i, channels[i] );

  // setup pointers to drawstuff callback functions
  fn.version = DS_VERSION; 
  fn.start = &start; 
  fn.step = &display; 
  fn.command = &command; 
  fn.stop = 0; 
#ifdef WIN32
  fn.path_to_textures = "C:/cga/kdc/walk02/useful/drawstuff-windows/textures";
#else
  fn.path_to_textures = "/home/cga/projects/useful/drawstuff/textures";
#endif

  // do display
  dsSimulationLoop( argc, argv, /* command line arguments */
                    500, 500, /* window size */
                    &fn ); /* callback info */

  return 0;
}

/********************************************************************/
