/*****************************************************************************/
/*
  data.c: save data from simulation
*/
/*****************************************************************************/
/*
*/
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "../useful/mrdplot/mrdplot.h"

#include "main.h"
#include "main2.h"

/*****************************************************************************/

extern SIM sim;

/*****************************************************************************/

enum channels {
M_TIME=0,

M_X,
M_XD,
M_X_D,
M_XD_D,
M_ANGLE,
M_ANGLED,
M_ANGLE_D,
M_ANGLED_D,
M_FORCE,
M_POLE_TOP_X,
M_POLE_TOP_Y,
M_POLE_TOP_Z,

N_CHANNELS
};

#define MAX_N_POINTS 100000

int n_points = 0;
float data[MAX_N_POINTS*N_CHANNELS];

char *names[N_CHANNELS];
char *units[N_CHANNELS];

/*****************************************************************************/

void init_data( SIM *s )
{
  int i;

  /*
  printf( "N_CHANNELS: %d; M_L_TORSO_BOTTOM: %d; last: %d\n", N_CHANNELS,
	  M_L_TORSO_BOTTOM, M_R_ANKLE_COMMAND );
  */

  for( i = 0; i < N_CHANNELS; i++ )
    {
      names[i] = "xxx";
      units[i] = "xxx";
    }

  names[M_TIME] = "time";

  names[M_X] = "x";
  names[M_XD] = "xd";
  names[M_X_D] = "x_d";
  names[M_XD_D] = "xd_d";
  names[M_ANGLE] = "angle";
  names[M_ANGLED] = "angled";
  names[M_ANGLE_D] = "angle_d";
  names[M_ANGLED_D] = "angled_d";
  
  names[M_FORCE] = "force";

  names[M_POLE_TOP_X] = "pole_x";
  names[M_POLE_TOP_Y] = "pole_y";
  names[M_POLE_TOP_Z] = "pole_z";

  units[M_TIME] = "s";

  units[M_X] = "m";
  units[M_XD] = "m/s";
  units[M_X_D] = "m";
  units[M_XD_D] = "m/s";
  units[M_ANGLE] = "r";
  units[M_ANGLED] = "r/s";
  units[M_ANGLE_D] = "r";
  units[M_ANGLED_D] = "r/s";
  
  units[M_FORCE] = "N";

  units[M_POLE_TOP_X] = "m";
  units[M_POLE_TOP_Y] = "m";
  units[M_POLE_TOP_Z] = "m";

  /*
  for( i = 0; i < N_CHANNELS; i++ )
    {
      printf( "%d %s %s\n", i, names[i], units[i] );
    }
  */
}

/*****************************************************************************/

int save_data( SIM *s )
{
  static int myindex = 0;

  if ( n_points >= MAX_N_POINTS )
    return 0;

  data[myindex + M_TIME] = s->time;

  data[myindex + M_X] = s->state[S_X];
  data[myindex + M_XD] = s->state[S_XD];
  data[myindex + M_X_D] = s->state_d[S_X];
  data[myindex + M_XD_D] = s->state_d[S_XD];
  data[myindex + M_ANGLE] = s->state[S_ANGLE];
  data[myindex + M_ANGLED] = s->state[S_ANGLED];
  data[myindex + M_ANGLE_D] = s->state_d[S_ANGLE];
  data[myindex + M_ANGLED_D] = s->state_d[S_ANGLED];
  data[myindex + M_FORCE] = s->controls_actual[U_FORCE];

  data[myindex + M_POLE_TOP_X] = s->locations[P_POLE_TOP][XX];
  data[myindex + M_POLE_TOP_Y] = s->locations[P_POLE_TOP][YY];
  data[myindex + M_POLE_TOP_Z] = s->locations[P_POLE_TOP][ZZ];

  myindex += N_CHANNELS;
  n_points++;

  return 0;
}

/*****************************************************************************/

int write_matlab_file( char *filename )
{
  FILE *stream;
  int i;
  static int myindex = 0;

  stream = fopen( filename, "w" );
  if ( stream == NULL )
    {
      fprintf( stderr, "Can't open %s for write.\n", filename );
      exit( -1 );
    }
  for ( i = 0; i < n_points; i++ )
    {
      fprintf( stream, "%g ", data[myindex + M_TIME] );
      fprintf( stream, "\n" );
      myindex += N_CHANNELS;
    }
  fclose( stream );

  return 0;
}

/*****************************************************************************/

int write_the_mrdplot_file( SIM *s )
{
  MRDPLOT_DATA *d; 

  d = malloc_mrdplot_data( 0, 0 );
  d->filename = generate_file_name();
  d->n_channels = N_CHANNELS;
  d->n_points = n_points;
  d->total_n_numbers = d->n_channels*d->n_points;
  d->frequency = 1/s->time_step;
  d->data = data;
  d->names = names;
  d->units = units;

  write_mrdplot_file( d );

  // write_matlab_file( "dd" );

  return 0;
}

/*****************************************************************************/
